@@ -7,6 +7,7 @@
 #include <WSCvdb.h>
 #include <WSCdb.h>
 #include <WSCsheetdef.h>
+#include <WSDkeyboard.h>
 
 #define WSsheet_VER                    1000
 
@@ -144,9 +145,6 @@ public:
     virtual void onMouseRelease(WSCpoint* pt);
     virtual void onVisibleChange(WSCbool vis);
 
-    WSMFguiClassDef(WSCsheet, WSCform);
-    WSMFversionDef(WSsheet);
-
 public:
     long      getRowCount() const { return _rows; }
     long      setRowCount(long rows = WSCSHEET_DEF_ROWS);
@@ -364,6 +362,7 @@ public:
     WSCstring getDbStmtString() { _dbwork = _dbStmtString; return _dbwork; }
 
     long      dbRefresh();
+    long      dataClear(long col = 2, long row = 2);
 
 protected:
     virtual   long _device_initialize();
@@ -374,6 +373,22 @@ protected:
     static    void onSheetMouseMove(WSCbase* obj);
     static    void onSheetMousePress(WSCbase* obj);
     static    void onSheetMouseRelease(WSCbase* obj);
+    virtual   void onScrMouseMove(WSCpoint* pt);
+    virtual   void onScrMousePress(WSCpoint* pt);
+    virtual   void onScrMouseRelease(WSCpoint* pt);
+    static    void onEditActivate(WSCbase* obj);
+    static    void onEditKeyPress(WSCbase* obj);
+    static    void onEditKeyRelease(WSCbase* obj);
+    static    void onEditKeyHook(WSCbase* obj);
+    static    void onEditValueChange(WSCbase* obj);
+    virtual   void onActivate();
+    virtual   void onKeyPress(WSDkeyboard* keyboard);
+    virtual   void onKeyRelease(WSDkeyboard* keyboard);
+    virtual   void onKeyHook(WSDkeyboard* keyboard);
+    virtual   void onValueChange();
+
+    WSMFguiClassDef(WSCsheet, WSCform);
+    WSMFversionDef(WSsheet);
 
     long      setCellText(const WSCstring& text);
     long      setCellText(long col, long row, const WSCstring& text);
@@ -472,7 +487,9 @@ protected:
     WSCuchar    _dbSort2;
     WSCuchar    _dbSort3;
     WSCstring   _dbwork;
-
+    WSCbool     _dbCellAutoResize;
+    WSCbool     _dbCellTitle;
+    
 protected:
     WSMFpropertyDef(WSNrows, long, _rows);
     WSMFpropertyDef(WSNcols, long, _cols);
@@ -505,6 +522,8 @@ protected:
     WSMFpropertyDef(WSNsort3, WSCuchar, _dbSort3);
     WSMFpropertyDef(WSNfilter, char*, _dbFilter);
     WSMFpropertyDef(WSNsql, char*, _dbStmtString);
+    WSMFpropertyDef(WSNdbCellAutoResize, WSCbool, _dbCellAutoResize);
+    WSMFpropertyDef(WSNdbCellTitle, WSCbool, _dbCellTitle);
 
 };
 #endif // _WSCSHEET_H
