@@ -246,6 +246,6 @@
   WSGIappLocaleSet()->setLocaleString("EUCJP","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("EUCJP","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("EUCJP","P3:WSSTR Perl(DBG)","Perl(DBG)");
-  WSGIappLocaleSet()->setLocaleString("EUCJP","f3:WSSTR Static option","DLL̵ӥ");
+  WSGIappLocaleSet()->setLocaleString("EUCJP","f3:WSSTR Static option","");
   WSGIappLocaleSet()->setLocaleString("EUCJP","l3:WSSTR Statically linked","ƥå(DLL ̵)");
   WSGIappLocaleSet()->setLocaleString("EUCJP","l3:WSSTR Dynamically linked","ʥߥå(DLL )");
