@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -69,6 +69,7 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
   _anchor_bottom_use = False;
   _anchor_left_use = False;
   _anchor_right_use = False;
+  _anchor_under_refreshing = False;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCnwbase);
@@ -77,6 +78,7 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
     WSMFpropertyCreate(WSNuserString,      char*,   _user_string,WSSuserString);
     WSMFpropertyCreate(WSNuserValue,       long,    _user_value, WSSuserValue);
     WSMFpropertyCreate(WSNblinkRefreshing, WSCbool, _bl_refreshing, WSSblinkRefreshing);
+    WSMFpropertySetSelection(WSRbool3, WSRbool3D);
     WSMFpropertyCreate(WSNvis,             WSCbool, _vis,WSSvis );
     WSMFpropertySetSelection(WSRbool2, WSRbool2D);
 
@@ -336,6 +338,8 @@ void WSCnwbase::setWorkWSNanchorRight(WSCushort data){
 void WSCnwbase::getWorkWSNanchorRight(WSCushort* data){}
 void WSCnwbase::setWorkWSNanchorTopFlag(WSCbool data){
   if (_anchor_top_use != False){
+    setGeometryChanged(True);
+    needUpdate();
     if (getParent() != NULL){
       getParent()->setGeometryChanged(True);
       getParent()->needUpdate();
@@ -345,6 +349,8 @@ void WSCnwbase::setWorkWSNanchorTopFlag(WSCbool data){
 void WSCnwbase::getWorkWSNanchorTopFlag(WSCbool* data){}
 void WSCnwbase::setWorkWSNanchorBottomFlag(WSCbool data){
   if (_anchor_bottom_use != False){
+    setGeometryChanged(True);
+    needUpdate();
     if (getParent() != NULL){
       getParent()->setGeometryChanged(True);
       getParent()->needUpdate();
@@ -354,6 +360,8 @@ void WSCnwbase::setWorkWSNanchorBottomFlag(WSCbool data){
 void WSCnwbase::getWorkWSNanchorBottomFlag(WSCbool* data){}
 void WSCnwbase::setWorkWSNanchorLeftFlag(WSCbool data){
   if (_anchor_left_use != False){
+    setGeometryChanged(True);
+    needUpdate();
     if (getParent() != NULL){
       getParent()->setGeometryChanged(True);
       getParent()->needUpdate();
@@ -363,6 +371,8 @@ void WSCnwbase::setWorkWSNanchorLeftFlag(WSCbool data){
 void WSCnwbase::getWorkWSNanchorLeftFlag(WSCbool* data){}
 void WSCnwbase::setWorkWSNanchorRightFlag(WSCbool data){
   if (_anchor_right_use != False){
+    setGeometryChanged(True);
+    needUpdate();
     if (getParent() != NULL){
       getParent()->setGeometryChanged(True);
       getParent()->needUpdate();
@@ -427,6 +437,7 @@ long WSCnwbase::execEventProc(long ev, void* data){
     }
     return WS_NO_ERR;
   }else if (ev == WSEV_MOUSE_PRESS  || ev == WSEV_MOUSE_RELEASE ||
+      ev == WSEV_MOUSE_DOUBLE_CLICK ||
       ev == WSEV_MOUSE_IN     || ev == WSEV_MOUSE_OUT    ||
       ev == WSEV_MOUSE_MOVE ){
 
@@ -458,7 +469,8 @@ long WSCnwbase::execEventProc(long ev, void* data){
           _mouse_enter = True;
           WSCbase::execEventProc(WSEV_MOUSE_IN, &vpdata);
         }
-        if (ev == WSEV_MOUSE_PRESS || ev == WSEV_MOUSE_RELEASE){
+        if (ev == WSEV_MOUSE_PRESS ||
+            ev == WSEV_MOUSE_DOUBLE_CLICK || ev == WSEV_MOUSE_RELEASE){
           return WS_NO_ERR;
         }
         return WS_ERR;
@@ -646,11 +658,15 @@ WSCbool WSCnwbase::getMouseGrabed(){
   return _mouse_grabed;
 }
 void WSCnwbase::_adjust_for_anchors(WSCushort w,WSCushort h){
+  WSCbool cleared = False;
   if (w != 0){
     if (_anchor_left_use == 0){
       if (_anchor_right_use != 0){
         long nx = w - _anchor_right - _w;
-        clear();
+        if (_x != nx){
+          clear();
+          cleared = True;
+        }
         _anchor_right_use = False;
         setProperty(WSNx,nx);
         _anchor_right_use = True;
@@ -659,20 +675,41 @@ void WSCnwbase::_adjust_for_anchors(WSCushort w,WSCushort h){
       if (_anchor_right_use != 0){
         _anchor_left_use = False;
         _anchor_right_use = False;
-        clear();
         if (w < _anchor_right + _anchor_left){
+          if (_w != 2){
+            clear();
+            cleared = True;
+          }
           setProperty(WSNwidth,2);
         }else{
           long val = w - _anchor_right - _anchor_left;
+          if (_x != _anchor_left){
+            clear();
+            cleared = True;
+          }
           setProperty(WSNx,_anchor_left);
           if (val > 5 ){
+            if (_w != val && cleared == False){
+              clear();
+              cleared = True;
+            }
             setProperty(WSNwidth,val);
           }else{
+            if (_w != 5 && cleared == False){
+              clear();
+              cleared = True;
+            }
             setProperty(WSNwidth,5);
           }
         }
         _anchor_left_use = True;
         _anchor_right_use = True;
+      }else{
+        if (_anchor_left != _x){
+          clear();
+          cleared = True;
+          setProperty(WSNx,_anchor_left);
+        }
       }
     }
   }
@@ -681,7 +718,10 @@ void WSCnwbase::_adjust_for_anchors(WSCushort w,WSCushort h){
     if (_anchor_top_use == 0){
       if (_anchor_bottom_use != 0){
         long ny = h - _anchor_bottom - _h;
-        clear();
+        if (ny != _h){
+          clear();
+          cleared = True;
+        }
         _anchor_bottom_use = False;
         setProperty(WSNy,ny);
         _anchor_bottom_use = True;
@@ -690,22 +730,50 @@ void WSCnwbase::_adjust_for_anchors(WSCushort w,WSCushort h){
       if (_anchor_bottom_use != 0){
         _anchor_top_use = False;
         _anchor_bottom_use = False;
-        clear();
         if (h < _anchor_bottom + _anchor_top){
+          if (5 != _h && cleared == False){
+            clear();
+            cleared = True;
+          }
           setProperty(WSNheight,5);
         }else{
           long val = h - _anchor_top - _anchor_bottom;
+          if (_y != _anchor_top && cleared == False){
+            clear();
+            cleared = True;
+          }
           setProperty(WSNy,_anchor_top);
           if (val > 5 ){
+            if (val != _h && cleared == False){
+              clear();
+              cleared = True;
+            }
             setProperty(WSNheight,val);
           }else{
+            if (5 != _h && cleared == False){
+              clear();
+              cleared = True;
+            }
             setProperty(WSNheight,5);
           }
         }
         _anchor_top_use = True;
         _anchor_bottom_use = True;
+      }else{
+        if (_anchor_top != _y && cleared == False){
+          cleared = True;
+          setProperty(WSNy,_anchor_top);
+        }
       }
     }
   }
+  if (cleared != False){
+    if (_anchor_under_refreshing != True){
+      _anchor_under_refreshing = True;
+      update();
+      _anchor_under_refreshing = False;
+    }
+    return;
+  }
 }
 
