@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -35,7 +35,7 @@
 #include <WSDcolor.h>
 #include <WSCfontSet.h>
 #include <WSDappDev.h>
-#include <WSClist.h>
+//#include <WSClist.h>
 
 WSMFguiClassInitialize(WSCvlabel, WSCnwbase);
 WSMFversion(WSCvlabel, WSCnwbase);
@@ -256,11 +256,14 @@ long WSCvlabel::_device_initialize(){
 }
 
 long WSCvlabel::draw(){
+dbprintf("WSCvlabel::draw %s:%d %s begin..\n",__FILE__,__LINE__,getInstanceName());
   if (getVisible() == False){
+dbprintf("WSCvlabel::draw %s:%d %s vis=False\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
   WSDdev* dev = getowndev();
   if (dev == NULL){
+dbprintf("WSCvlabel::draw %s:%d %s dev=NULL\n",__FILE__,__LINE__,getInstanceName());
     return WS_ERR;
   }
 
@@ -271,6 +274,7 @@ long WSCvlabel::draw(){
 
   WSCbool absolute = getAbsoluteDraw();
   if (absolute == False && dev->isExposed(x, y, w, h) == False){
+dbprintf("WSCvlabel::draw %s:%d %s not exposed..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
 
@@ -295,6 +299,7 @@ long WSCvlabel::draw(){
   }
   if ( shadowtype == WS_SHADOW_NONE &&
         bldraw && _tw_fl == False){
+dbprintf("WSCvlabel::draw %s:%d %s not draw..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
 
@@ -309,6 +314,7 @@ long WSCvlabel::draw(){
 
   long err = dev->beginDraw(x, y, w, h, absolute);
   if (err != WS_NO_ERR){
+dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceName());
      return WS_NO_ERR;
   }
 
@@ -450,9 +456,11 @@ long WSCvlabel::draw(){
         }else{
           WSCstring sep(",");
           if (getParent() != NULL){
-            WSClist* list = (WSClist*)getParent()->cast("WSClist");
+//            WSClist* list = (WSClist*)getParent()->cast("WSClist");
+            void* list = getParent()->cast("WSClist");
             if (list != NULL){
-              sep = list->getSeparator();
+//              sep = list->getSeparator();
+              sep = getParent()->getProperty(WSNseparator);
             }
           }
           long num2 = _bar_pos.getWords(sep);
@@ -612,9 +620,11 @@ long WSCvlabel::draw(){
     }else{
       WSCstring sep(",");
       if (getParent() != NULL){
-        WSClist* list = (WSClist*)getParent()->cast("WSClist");
+//        WSClist* list = (WSClist*)getParent()->cast("WSClist");
+        void* list = getParent()->cast("WSClist");
         if (list != NULL){
-          sep = list->getSeparator();
+//          sep = list->getSeparator();
+          sep = getParent()->getProperty(WSNseparator);
         }
       }
       long num2 = _bar_pos.getWords(sep);
@@ -669,3 +679,21 @@ static long table[3][3];
 char* WSCvlabel::getDefaultProperty(){
   return WSNlabelString;
 }
+WSCuchar WSCvlabel::get_vk_draw(){
+  return _vk_draw;
+}
+void WSCvlabel::set_vk_draw(WSCuchar val){
+  _vk_draw = val;
+}
+WSCbool WSCvlabel::get_use_icon(){
+  return _use_icon;
+}
+void WSCvlabel::set_use_icon(WSCbool val){
+  _use_icon = val;
+}
+WSCstring WSCvlabel::get_bar_pos(){
+  return _bar_pos;
+}
+void WSCvlabel::set_bar_pos(WSCstring val){
+  _bar_pos = val;
+}
