@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -177,6 +177,7 @@ long WSDwindowDev::setValue(long kind,void* data){
 
 void WSDwindowDev::onFocusChange(){
   if (getAttachedClient() != NULL){
+//printf("WSDwindowDev::onFocusChange..\n");
     getAttachedClient()->setFocus();
   }
 }
@@ -522,7 +523,12 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
   _event_delivering_sequence = True;
   _client_deleted = False;
   long need_execute_later = 0;
+//  long exclusive_mask = WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_DOUBLE_CLICK_BIT;
   long exclusive_mask = WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_RELEASE_BIT;
+
+//long t1 = WSGFclocktime(); //TEST
+//printf("WSDwindowDev::execEvent ev=%d %s\n",ev_kind,getAttachedClient()->getInstanceName());
+
   for(i=0; i < num; i++){
     WSDdev* dev = array[i];
     if (dev == this && (ev_bit & exclusive_mask)){
@@ -553,7 +559,7 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
       }
     }
   }
-
+//printf("WSDwindowDev::execEvent t1=%d %s\n",WSGFclocktime() - t1,getAttachedClient()->getInstanceName());
   WSClistData* list = NULL;
   if (need_execute_later != 0){
     long deleted = 0;
@@ -583,6 +589,7 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
 
   delete array;
   _event_delivering_sequence = False;
+//printf("WSDwindowDev::execEvent t2=%d %s\n",WSGFclocktime() - t1,getAttachedClient()->getInstanceName());
   return WS_NO_ERR;
 }
 
@@ -805,6 +812,7 @@ WSCbool WSDwindowDev::isAvailableEvent(long ev_kind){
      case  WSEV_MOUSE_OUT:    return True;
      case  WSEV_MOUSE_PRESS:    return True;
      case  WSEV_MOUSE_RELEASE:  return True;
+     case  WSEV_MOUSE_DOUBLE_CLICK:  return True;
      case  WSEV_INITIALIZE:   return True;
      case  WSEV_VISIBLE_CH:   return True;
      case  WSEV_KEY_PRESS:    return True;
