@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -68,6 +68,9 @@ void libws_func(){
 #ifndef BTRON
 FILE* _ws_debug_fp = NULL;
 #else
+#ifdef TE
+#include <btron/bsocket.h>
+#endif
 FILE* _ws_debug_fp = NULL;
 //FILE* _ws_debug_fp = stdout;
 #endif
@@ -2243,6 +2246,9 @@ long WSGFnameToEvent(char* ev){
   if (!WSGFstrcmp(ev,"WSEV_MOUSE_MOVE")){
      return WSEV_MOUSE_MOVE;
   }else
+  if (!WSGFstrcmp(ev,"WSEV_MOUSE_DOUBLE_CLICK")){
+     return WSEV_MOUSE_DOUBLE_CLICK;
+  }else
   if (!WSGFstrcmp(ev,"WSEV_INITIALIZE")){
      return WSEV_INITIALIZE;
   }else
@@ -3052,8 +3058,7 @@ void WSGFcreateVerticalString(WSCstring* str){
   return;
 }
 
-char* WSGFgetFileName(char* path){
-static char buf[1024];
+WSCstring WSGFgetFileName(char* path){
   WSCstring tmp(path);
   tmp.delHeadSpace();
   tmp.delTailSpace();
@@ -3066,28 +3071,23 @@ static char buf[1024];
 
   if (num == 1){
 #ifndef MSW
-    strcpy(buf,tmp.getString());
-    return buf;
+    return tmp;
 #else
     char* sep =":";
     long num = tmp.getWords(sep);
     if (num == 1){
-      strcpy(buf,tmp.getString());
-      return buf;
+      return tmp;
     }
     WSCstring filename = tmp.getWord(num -1,sep);
-    strcpy(buf,filename.getString());
-    return buf;
+    return filename;
 #endif
   } 
 
   WSCstring filename = tmp.getWord(num-1,sep);
-  strcpy(buf,filename.getString());
-  return buf;
+  return filename;
 }
 
-char* WSGFgetDirName(char* path){
-static char buf[1024];
+WSCstring WSGFgetDirName(char* path){
   WSCstring tmp(path);
   tmp.delHeadSpace();
   tmp.delTailSpace();
@@ -3110,8 +3110,7 @@ static char buf[1024];
   }else{
     tmp.cutString(pos);
   }
-  strcpy(buf,tmp.getString());
-  return buf;
+  return tmp;
 }
 
 long WSGFloadWindow(char* stype,char* fname,WSCbase** btarget,WSCbase* parent){
