@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -31,12 +31,15 @@
 #include <WSDdev.h>
 #include <WSDwindowDev.h>
 #include <WSDprivateTimer.h>
+#include <WSCwindow.h>
 
 WSMFguiClassInitialize(WSCsform,WSCform);
 WSMFversion(WSCsform,WSCform);
 WSMFdefineUseDevice(WSCsform,formDev);
 
 WSDprivateTimer* _delay_timer = NULL;
+WSDprivateTimer*  _move_timer = NULL;
+WSCwindow* _sep_window = NULL;
 
 WSCsform::WSCsform(WSCbase* base,char* objname):
                 WSCform(base,objname){
@@ -103,7 +106,18 @@ void WSCsform::setWorkWSNminimum(WSCuchar){
   _adjust();
 }
 void WSCsform::getWorkWSNminimum(WSCuchar*){ }
-void WSCsform::setWorkWSNrefreshing(WSCbool){ }
+void WSCsform::setWorkWSNrefreshing(WSCbool){
+  if (_move_timer != NULL){
+    void* data = _move_timer->getData();
+    if (data != (void*)this){
+      if (_sep_window != NULL &&
+          _sep_window->getVisible() != False){
+        _sep_window->setVisible(False);
+      }
+      _move_timer->stopTimer();
+    }
+  }
+}
 void WSCsform::getWorkWSNrefreshing(WSCbool*){ }
 void WSCsform::setWorkWSNbarValue(char* pos){
   _parse_pos(pos);
@@ -186,12 +200,15 @@ void WSCsform::setWorkWSNwidth(WSCushort w){
       _need_adjust = True;
 //      _adjust();
     }
+    _adjust_pos();
   }
+  _need_adjust = True;
 }
 void WSCsform::setAdjustChildren(WSCbool fl){
   _adjust_btn = fl;
 }
 void WSCsform::setWorkWSNheight(WSCushort h){
+  WSCform::setWorkWSNheight(h);
   if (_orientation == WS_HORIZONTAL){
     long num = (long)_pos_list.getNum();
     if (h < _shadow_thick*2 + _margin_top + _margin_bottom + _bw * num + 2){
@@ -209,8 +226,28 @@ void WSCsform::setWorkWSNheight(WSCushort h){
         }
       }
     }
+
+    long first = 1;
+    WSCstring str;
+    for(i=0; i<num; i++){
+      long val = (long)_pos_list[i];
+      if (first == 0){
+        str += _separator;
+      }else{
+        first = 0;
+      }
+      str += WSGFltoa(val);
+    }
+    if (num != 0 ){
+      if (_pos != NULL){
+        delete _pos;
+      }
+      _pos = WSGFstrdup((char*)str);
+      _need_adjust = True;
+//      _adjust();
+    }
+    _adjust_pos();
   }
-  WSCform::setWorkWSNheight(h);
   _need_adjust = True;
 //  _adjust();
 }
@@ -331,11 +368,28 @@ void WSCsform::onMousePress(WSCpoint* pt){
     }
   }
   WSCform::onMousePress(pt);
+  if (_pressed != False && _refreshing == False){
+    if (_move_timer == NULL){
+      _move_timer = WSDprivateTimer::getNewInstance();
+      _move_timer->setRate(20);
+    }
+    _move_timer->setWorkProc(_move_adjust_work,this);
+    _move_timer->startTimer();
+  }
 }
 void WSCsform::onValueChange(){
   execProcedure(WSEV_VALUE_CH);
 }
 void WSCsform::onMouseRelease(WSCpoint* pt){
+  if (_pressed != False && _refreshing == False){
+    if (_sep_window != NULL &&
+        _sep_window->getVisible() != False){
+      _sep_window->setVisible(False);
+    }
+    if (_move_timer != NULL){
+      _move_timer->stopTimer();
+    }
+  }
   _pressed = False;
   WSCform::onMouseRelease(pt);
   _exec_resize = 1;
@@ -363,6 +417,56 @@ void WSCsform::_clear_area(){
   dev->endDraw();
 }
 
+void WSCsform::_move_adjust_work(void* ptr){
+  WSCsform* sform = (WSCsform*)ptr;
+  if (sform->_pressed == False){
+    if (_sep_window != NULL &&
+        _sep_window->getVisible() != False){
+      _sep_window->setVisible(False);
+    }
+    _move_timer->stopTimer();
+    return;
+  }
+#ifdef TE
+  WSCushort tmpv = sform->_bw; //for compiler bug..
+#endif
+  long pos = (long)sform->_pos_list[sform->_hit_bar];
+  WSDdev* dev = sform->getowndev();
+  if (dev == NULL){
+    return;
+  }
+
+  short px,py;
+  WSCushort pw = sform->_w;
+  WSCushort ph = sform->_h;
+
+  dev->getDispAddr(&px,&py);
+  if (sform->_orientation == WS_VERTICAL){
+    px += (long)sform->_pos_list[sform->_hit_bar] - sform->_bw/2;
+    pw = sform->_bw;
+  }else{
+    py += (long)sform->_pos_list[sform->_hit_bar] - sform->_bw/2;
+    ph = sform->_bw;
+  }
+  if (_sep_window == NULL){
+    _sep_window = new WSCwindow(NULL,"sform_sep_window");
+    _sep_window->initialize();
+    _sep_window->setProperty(WSNtitleBar,WS_NO_MANAGE);
+    _sep_window->setProperty(WSNshadowThickness,0);
+    _sep_window->setProperty(WSNbackColor,"#606060");
+  }
+  if (_sep_window == NULL){
+    _move_timer->stopTimer();
+    return;
+  }
+  _sep_window->setProperty(WSNx,px);
+  _sep_window->setProperty(WSNy,py);
+  _sep_window->setProperty(WSNwidth,pw);
+  _sep_window->setProperty(WSNheight,ph);
+  if (_sep_window->getVisible() == False){
+    _sep_window->setVisible(True);
+  }
+}
 void WSCsform::_delay_adjust_work(void* ptr){
   WSCsform* sform = (WSCsform*)ptr;
   sform->_need_adjust = True;
@@ -424,22 +528,24 @@ void WSCsform::onMouseMove(WSCpoint* pt){
       delete _pos;
     }
     _pos = WSGFstrdup((char*)str); 
+    if (_refreshing != False){
 #ifndef MSW
-    if (_delay_adjust_flag == 0){
-      if (_delay_timer == NULL){
-        _delay_timer = WSDprivateTimer::getNewInstance();
-        _delay_timer->setRate(50);
-        _delay_timer->setCont(False);
+      if (_delay_adjust_flag == 0){
+        if (_delay_timer == NULL){
+          _delay_timer = WSDprivateTimer::getNewInstance();
+          _delay_timer->setRate(50);
+          _delay_timer->setCont(False);
+        }
+        _delay_timer->setWorkProc(_delay_adjust_work,this);
+        _delay_timer->startTimer();
       }
-      _delay_timer->setWorkProc(_delay_adjust_work,this);
-      _delay_timer->startTimer();
-    }
 #else //MSW
-    _need_adjust = True;
-    _adjust();
-    _clear_area();
+      _need_adjust = True;
+      _adjust();
+      _clear_area();
 //    redraw();
 #endif //MSW
+    }
   }else{
     WSDdev* dev = getowndev();
     if (dev != NULL){
@@ -523,7 +629,12 @@ void WSCsform::update(){
 }
 
 void WSCsform::onResize(WSCrect* area){
+  WSCushort w = _w;
+  WSCushort h = _h;
   WSCform::onResize(area);
+  if (w != _w || h != _h){
+    _need_adjust = True;
+  }
   if (_need_adjust != False){
     _adjust();
   }
@@ -557,7 +668,45 @@ WSCbase* _get_ext_child(WSClistData* children,long no){
       }
     }
 }
-
+void WSCsform::_adjust_pos(){
+  long num = _pos_list.getNum()+1;
+  long i;
+  long err = False;
+//printf("WSCsform::_adjust_pos %s num=%d _min=%d w,h=%d,%d\n",getInstanceName(),num,_minimum,_w,_h);
+  for(i=0; i<num; i++){
+    long pre = _margin_left;
+    if (_orientation != WS_VERTICAL){
+      pre = _margin_top;
+    }
+    if (i > 0){
+      pre = (long)_pos_list.getData(i-1);
+    }
+    long pos = (long)_pos_list.getData(i);
+    if (i == num-1){
+      if (_orientation == WS_VERTICAL){
+        pos = _w - _margin_right;
+      }else{
+        pos = _h - _margin_bottom;
+      }
+    }
+//printf("WSCsform::_adjust_pos i=%d dif=%d pre=%d pos=%d\n",i,pos - pre,pre,pos);
+    if (pos - pre < _minimum){
+     err = True;
+     break;
+    }
+  }
+  if (err != False && num > 1){
+    long val = (_w - _margin_left - _margin_right) / num;
+    if (_orientation != WS_VERTICAL){
+      val = (_h - _margin_left - _margin_right) / num;
+    }
+    for(i=0; i<num-1; i++){
+      _pos_list.setData(i,(void*)(val*(i+1)));
+//printf("WSCsform::_adjust_pos i=%d val=%d\n",i,val*(i+1));
+    }
+    _need_adjust = True;
+  }
+}
 void WSCsform::_adjust(){
   if (_need_adjust == False){
     return;
@@ -703,6 +852,13 @@ void WSCsform::_adjust(){
     }
   }
 }
+WSCbool WSCsform::getManaged(WSCbase* target){
+  long nomanaged = (long)target->getUserData("sform-managed"); 
+  if (nomanaged == 0){
+    return True;
+  }
+  return False;
+}
 long WSCsform::setManaged(WSCbase* target,WSCbool fl){
   WSCbase* parent = target->getParent();
   if (parent != this){
@@ -721,6 +877,7 @@ long WSCsform::setManaged(WSCbase* target,WSCbool fl){
     target->setUserData("sform-managed",(void*)False); 
   }
   _need_adjust = True;
+  _adjust_pos();
   _adjust();
   return WS_NO_ERR;
 }
