@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -32,6 +32,7 @@
 #include <WSDtimer.h>
 #include <WSDmouse.h>
 #include <WSDwindowDev.h>
+#include <WSCfontSet.h>
 
 WSMFguiClassInitialize(WSCfform,WSCbase);
 WSMFversion(WSCfform,WSCbase);
@@ -73,18 +74,23 @@ WSCfform::WSCfform(WSCbase* base,char* objname):WSCbase(base,objname){
   _mouse_x = 0;
   _mouse_y = 0;
   _mouse_pressed = False;
+  _mouse_pressed_resize = False;
   _floated = False;
+  _indicator = True;
 //  _dev_form = NULL;
 //  _dev_win = NULL;
   _title_string = WSGFstrdup("");
   _change_sequence = False;
   _orientation = WS_VERTICAL;
+  _bar_thick = 8;
+  _font = (char)WSGIappFontSet()->getDefaultFontNo();
+  _margin = 0;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCfform);
     WSMFpropertyCreate(WSNname,  char*, _instance_name,WSSname );
     WSMFpropertyCreate(WSNuserString,  char*, _user_string,WSSuserString );
-    WSMFpropertyCreate(WSNtitleString, char*, _title_string,WSSuserString );
+    WSMFpropertyCreate(WSNtitleString, char*, _title_string,WSStitleString );
     WSMFpropertyCreate(WSNuserValue,   long,  _user_value,WSSuserValue  );
     WSMFpropertyCreate(WSNx,           short,   _x ,WSSx );
     WSMFpropertyCreate(WSNy,           short,   _y ,WSSy );
@@ -128,7 +134,11 @@ WSCfform::WSCfform(WSCbase* base,char* objname):WSCbase(base,objname){
 
     WSMFpropertyCreate(WSNexport,WSCbool,_export_,WSSexport);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
-
+    WSMFpropertyCreate( WSNindicatorOn,  WSCbool, _indicator,WSSindicatorOn);
+      WSMFpropertySetSelection(WSRbool3,WSRbool3D);
+    WSMFpropertyCreate( WSNbarThickness,  WSCuchar, _bar_thick,WSSbarThickness); 
+    WSMFpropertyCreate(WSNfont,         WSCuchar,   _font,WSSfont  );
+    WSMFpropertyCreate(WSNmargin,WSCuchar,_margin,WSSmargin);
 
     WSMFaddTrigger(WSEV_INITIALIZE  );
     WSMFaddTrigger(WSEV_MOUSE_MOVE  );
@@ -168,17 +178,22 @@ WSMFproperty(WSCfform,WSNpixmapStyle, WSCuchar, _pixmap_style ,WS_DIRECT_WINDOW)
 WSMFproperty(WSCfform, WSNmouse,   WSCushort,     _mouse_no     ,0);
 WSMFproperty(WSCfform, WSNgradation, WSCuchar, _grad_no   ,0);
 WSMFproperty(WSCfform, WSNgradationMargin, WSCuchar, _grad_margin ,0);
-WSMFproperty( WSCfform, WSNanchorTop,WSCushort,_anchor_top,0 );
-WSMFproperty( WSCfform, WSNanchorBottom,WSCushort,_anchor_bottom,0 );
-WSMFproperty( WSCfform, WSNanchorLeft,WSCushort,_anchor_left,0 );
-WSMFproperty( WSCfform, WSNanchorRight,WSCushort,_anchor_right,0 );
-WSMFproperty( WSCfform, WSNanchorTopFlag,WSCbool,_anchor_top_use,False );
-WSMFproperty( WSCfform, WSNanchorBottomFlag,WSCbool,_anchor_bottom_use,False );
-WSMFproperty( WSCfform, WSNanchorLeftFlag,WSCbool,_anchor_left_use,False );
-WSMFproperty( WSCfform, WSNanchorRightFlag,WSCbool,_anchor_right_use,False );
-WSMFproperty( WSCfform, WSNorientation,WSCuchar,_orientation,WS_VERTICAL );
-WSMFproperty( WSCfform, WSNexport,WSCbool,_export_,False );
-
+WSMFproperty(WSCfform, WSNanchorTop,WSCushort,_anchor_top,0 );
+WSMFproperty(WSCfform, WSNanchorBottom,WSCushort,_anchor_bottom,0 );
+WSMFproperty(WSCfform, WSNanchorLeft,WSCushort,_anchor_left,0 );
+WSMFproperty(WSCfform, WSNanchorRight,WSCushort,_anchor_right,0 );
+WSMFproperty(WSCfform, WSNanchorTopFlag,WSCbool,_anchor_top_use,False );
+WSMFproperty(WSCfform, WSNanchorBottomFlag,WSCbool,_anchor_bottom_use,False );
+WSMFproperty(WSCfform, WSNanchorLeftFlag,WSCbool,_anchor_left_use,False );
+WSMFproperty(WSCfform, WSNanchorRightFlag,WSCbool,_anchor_right_use,False );
+WSMFproperty(WSCfform, WSNorientation,WSCuchar,_orientation,WS_VERTICAL );
+WSMFproperty(WSCfform, WSNexport,WSCbool,_export_,False );
+WSMFproperty(WSCfform, WSNindicatorOn,  WSCbool,  _indicator   ,True);
+WSMFproperty(WSCfform, WSNbarThickness,  WSCuchar,  _bar_thick   ,8);
+WSMFproperty(WSCfform, WSNfont, WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
+WSMFproperty(WSCfform, WSNmargin, WSCuchar, _margin ,0);
+void WSCfform::setWorkWSNfont(WSCuchar){}
+void WSCfform::getWorkWSNfont(WSCuchar*){}
 void WSCfform::setWorkWSNorientation(WSCuchar ){}
 void WSCfform::getWorkWSNorientation(WSCuchar* ){}
 void WSCfform::setWorkWSNname(char* ){}
@@ -252,11 +267,15 @@ void WSCfform::setWorkWSNwidth(WSCushort data){
     }
     _dev->setValue(WSDEV_WIDTH,(void*)&data);
     _geometry_changed = True;
+//printf("WSCfform::setWorkWSNwidth %s ->%d\n",getInstanceName(),_w);
     _update_resize = True;
     if (_grad_no != 0){
       setAbsoluteDraw(True);
       needUpdate();
     }
+  }else{
+    _geometry_changed = True;
+    _update_resize = True;
   }
 }
 void WSCfform::getWorkWSNwidth(WSCushort* ){}
@@ -280,6 +299,9 @@ void WSCfform::setWorkWSNheight(WSCushort data){
       setAbsoluteDraw(True);
       needUpdate();
     }
+  }else{
+    _geometry_changed = True;
+    _update_resize = True;
   }
 }
 void WSCfform::getWorkWSNheight(WSCushort* ){}
@@ -322,7 +344,11 @@ void WSCfform::setWorkWSNdet(WSCbool data){
     }
     setSensitive(data);
   }else{
-    setUserData(WS_DET,(void*)data);
+    if (data == False){
+      setUserData(WS_DET,(void*)0);
+    }else{
+      setUserData(WS_DET,(void*)1);
+    }
   }
 }
 void WSCfform::getWorkWSNdet(WSCbool* data){
@@ -352,6 +378,8 @@ void WSCfform::setWorkWSNgradation(WSCuchar ){}
 void WSCfform::getWorkWSNgradation(WSCuchar* ){}
 void WSCfform::setWorkWSNgradationMargin(WSCuchar ){}
 void WSCfform::getWorkWSNgradationMargin(WSCuchar* ){}
+void WSCfform::setWorkWSNmargin(WSCuchar ){}
+void WSCfform::getWorkWSNmargin(WSCuchar* ){}
 
 void WSCfform::setWorkWSNanchorTop(WSCushort data){
   if (_anchor_top_use != False){
@@ -431,6 +459,11 @@ void WSCfform::setWorkWSNexport(WSCbool data){
 }
 void WSCfform::getWorkWSNexport(WSCbool* data){
 }
+void WSCfform::setWorkWSNindicatorOn(WSCbool data){}
+void WSCfform::getWorkWSNindicatorOn(WSCbool* data){}
+void WSCfform::setWorkWSNbarThickness(WSCuchar data){}
+void WSCfform::getWorkWSNbarThickness(WSCuchar* data){}
+
 
 WSCbool WSCfform::getNoRefresh(){
   return _no_refresh;
@@ -535,7 +568,7 @@ long WSCfform::_device_initialize(){
     return WS_ERR;
   }
   if (_floated == False){
-    char pstyle = _pixmap_style;
+    char pstyle = getPixmapStyle();
     dev->setVisible(False);
     if (getXOffsetPtr() == NULL){
       dev->setValue(WSDEV_X,(void*)&_x);
@@ -568,6 +601,7 @@ long WSCfform::_device_initialize(){
 //        _dev->setValue(WSDEV_TITLE_BAR,(void*)&val);
 //      }
     }
+
 //    if (_title_bar == WS_NO_MANAGE && getPropertyEditMode() == False){
         WSCbool fl = True;
         dev->setValue(WSDEV_NO_FRAME,(void*)&fl);
@@ -585,7 +619,7 @@ long WSCfform::_device_initialize(){
     dev->setValue(WSDEV_BACKCOLOR,(void*)&_back_color);
     dev->setValue(WSDEV_BACK_PIXMAP,(void*)&_bp_no);
   
-    char pixmap_style = _pixmap_style;
+    char pixmap_style = getPixmapStyle();
     dev->setValue(WSDEV_PIXMAP_STYLE,(void*)&pixmap_style);
     if (_title_string != NULL){
       dev->setValue(WSDEV_TITLE_STRING,(void*)_title_string);
@@ -643,6 +677,10 @@ long WSCfform::draw(){
     }
 #endif
   }
+  if (_update_resize != False){
+    _update_resize = False;
+    execResizeEvent();
+  }
 
   if (_geometry_changed != False){
     _exec_child_adjust_for_anchors();
@@ -685,29 +723,63 @@ long WSCfform::draw(){
   }
   WSGFdrawShadow(dev,_shadow_type,_shadow_thick,_fore_color,_ts_color,
                  _bs_color,_back_color,0,0,_w,_h);
+
+  long swidth = 0;
+  WSCstring str(_title_string);
+  if (_title_string[0] != 0){
+    swidth = WSGIappFontSet()->getStringWidth(_font,&str);
+  }
+
   if (_orientation == WS_VERTICAL){
-    if ( _h - _shadow_thick*2 - 8){
+    if ( _h - _shadow_thick*2 - 8 - _margin > 0){
       WSGFdrawShadow(dev,WS_SHADOW_OUT,1,_fore_color,_ts_color,
                    _bs_color,_back_color,
-       _shadow_thick+2 ,_shadow_thick + 2,2,_h - _shadow_thick*2 - 8);
+       _shadow_thick + _bar_thick/2 -2 ,_shadow_thick + 2,2,_h - _shadow_thick*2 - 8 - _margin);
       WSGFdrawShadow(dev,WS_SHADOW_OUT,1,_fore_color,_ts_color,
                    _bs_color,_back_color,
-       _shadow_thick+6 ,_shadow_thick + 2,2,_h - _shadow_thick*2 - 8);
+       _shadow_thick + _bar_thick/2 +2 ,_shadow_thick + 2,2,_h - _shadow_thick*2 - 8 - _margin);
     }
   }else{
-    if ( _w - _shadow_thick*2 - 8){
+    if ( _w - _shadow_thick*2 - 8 - _margin - swidth > 0){
       WSGFdrawShadow(dev,WS_SHADOW_OUT,1,_fore_color,_ts_color,
                    _bs_color,_back_color,
-       _shadow_thick+2 ,_shadow_thick + 2,_w - _shadow_thick*2 - 8,2);
+       _shadow_thick+2 + swidth, _shadow_thick + _bar_thick/2 - 2,_w - _shadow_thick*2 - 8 - swidth - _margin, 2);
       WSGFdrawShadow(dev,WS_SHADOW_OUT,1,_fore_color,_ts_color,
                    _bs_color,_back_color,
-       _shadow_thick+2 ,_shadow_thick + 6,_w - _shadow_thick*2 - 8,2);
+       _shadow_thick+2 + swidth, _shadow_thick + _bar_thick/2 + 2,_w - _shadow_thick*2 - 8 - swidth - _margin, 2);
+    }
+    if (_title_string[0] != 0 && _orientation != WS_VERTICAL){
+      dev->setForeColor(_fore_color);
+      long fh = WSGIappFontSet()->getStringHeight(_font,&str);
+      dev->drawString(_shadow_thick+2,_shadow_thick+2,swidth,fh,&str,_font,WS_CENTER,-1,False);
     }
   }
+
+
+  if (getFloated() != False && _indicator != False){
+    long margin = 2 + _shadow_thick;
+    dev->setLineWidth(0);
+    dev->setForeColor(_ts_color);
+    dev->drawLine(_w - 15 - margin,_h -3 ,_w -3,_h -15 - margin);
+    dev->drawLine(_w - 11 - margin,_h -3 ,_w -3,_h -11 - margin);
+    dev->drawLine(_w - 7 - margin,_h -3 ,_w -3,_h -7 - margin);
+    dev->drawLine(_w - 3 - margin,_h -3 ,_w -3,_h -3 - margin);
+    dev->setForeColor(_bs_color);
+    dev->drawLine(_w - 14 - margin,_h -3 ,_w -3,_h -14 - margin);
+    dev->drawLine(_w - 13 - margin,_h -3 ,_w -3,_h -13 - margin);
+    dev->drawLine(_w - 10 - margin,_h -3 ,_w -3,_h -10 - margin);
+    dev->drawLine(_w - 9 - margin,_h -3 ,_w -3,_h -9 - margin);
+    dev->drawLine(_w - 6 - margin,_h -3 ,_w -3,_h -6 - margin);
+    dev->drawLine(_w - 5 - margin,_h -3 ,_w -3,_h -5 - margin);
+    dev->drawLine(_w - 2 - margin,_h -3 ,_w -3,_h -2 - margin);
+    dev->drawLine(_w - 1 - margin,_h -3 ,_w -3,_h -1 - margin);
+  }
+
   dev->endDraw();
   return WS_NO_ERR;
 }
 void WSCfform::_exec_child_adjust_for_anchors(){
+#if 0
   if (_geometry_changed != False){
     WSClistData children = getChildren();
     long num = children.getNum();
@@ -717,6 +789,27 @@ void WSCfform::_exec_child_adjust_for_anchors(){
       child->_adjust_for_anchors(_w,_h);
     } 
   }
+#endif
+  if (adjustAnchor() != False){
+    if (_geometry_changed != False){
+      WSClistData children = getChildren();
+      long num = children.getNum();
+      long i;
+      for(i=0; i<num; i++){
+        WSCbase* child = (WSCbase*)children[i];
+        child->_adjust_for_anchors(_w,_h);
+        if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
+          if (child->isNeedUpdate() != False){
+            if ((long)child->getUserData("anchor update..") == 0){
+              child->setUserData("anchor update..",(void*)1);
+              child->update();
+              child->setUserData("anchor update..",(void*)0);
+            }
+          }
+        }
+      }
+    }
+  }
 }
 
 void WSCfform::update(){
@@ -792,7 +885,7 @@ long WSCfform::cdraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-  if (_pixmap_style != WS_DIRECT_WINDOW){
+  if (getPixmapStyle() != WS_DIRECT_WINDOW){
     redraw();
   }else{
     WSCrect area; 
@@ -839,6 +932,10 @@ void WSCfform::_adjust_for_anchors(WSCushort w,WSCushort h){
         }
         _anchor_left_use = True;
         _anchor_right_use = True;
+      }else{
+        if (_anchor_left != _x){
+          setProperty(WSNx,_anchor_left);
+        }
       }
     }
     needUpdate();
@@ -865,6 +962,10 @@ void WSCfform::_adjust_for_anchors(WSCushort w,WSCushort h){
         }
         _anchor_top_use = True;
         _anchor_bottom_use = True;
+      }else{
+        if (_anchor_top != _y){
+          setProperty(WSNy,_anchor_top);
+        }
       }
     }
     needUpdate();
@@ -875,27 +976,76 @@ void WSCfform::onMousePress(WSCpoint* pt){
   WSCbase::onMousePress(pt);
   if (_orientation == WS_VERTICAL){
     if (pt->x > 0 && pt->x < 15 &&
-        pt->y > 0 && pt->y < _h){
+        pt->y > 0 && pt->y < _h - _margin){
       _mouse_x = pt->x;
       _mouse_y = pt->y;
       _mouse_pressed = True;
     }
   }else{
-    if (pt->x > 0 && pt->x < _w &&
+    if (pt->x > 0 && pt->x < _w  - _margin &&
         pt->y > 0 && pt->y < 15){
       _mouse_x = pt->x;
       _mouse_y = pt->y;
       _mouse_pressed = True;
     }
   }
+  if (_floated != False){
+    if (pt->x > _w -15 && pt->x < _w &&
+        pt->y > _h -15 && pt->y < _h){
+      if (_indicator != False){
+        _mouse_pressed_resize = True;
+        if (_resize_timer == NULL){
+          _resize_timer = WSDprivateTimer::getNewInstance();
+        }
+        _resize_timer->stopTimer();
+        _resize_timer->setWorkProc(_resize_proc,this);
+        _resize_timer->setRate(50);
+        _resize_timer->setCont(True);
+        _resize_timer->startTimer();
+      }
+    }
+  }
+}
+
+void WSCfform::_resize_proc(void* ptr){
+  WSCfform* _this = (WSCfform*)ptr;
+
+  short x,y;
+  WSGIappMouse()->getMousePosition(&x,&y,_this);
+  WSCulong status = WSGIappMouse()->getMouseStatus();
+
+  if (_this->_mouse_pressed_resize != False){
+    if (x > 20){
+      _this->setProperty(WSNwidth,x + 5);
+    }
+    if (y > 20){
+      _this->setProperty(WSNheight,y + 5);
+    }
+    _this->update();
+
+    if (status & ( WS_MOUSE_BTN1 | WS_MOUSE_BTN2 | WS_MOUSE_BTN3)){
+    }else{
+      _resize_timer->stopTimer();
+      _this->_mouse_pressed_resize = False;
+    }
+
+  }else{
+    _resize_timer->stopTimer();
+    _this->_mouse_pressed_resize = False;
+  }
+
 }
 void WSCfform::onMouseMove(WSCpoint* pt){
   WSCbase::onMouseMove(pt);
-  if (_mouse_pressed == False){
+//printf("move.. %s\n",getInstanceName());
+  if (_mouse_pressed == False && _mouse_pressed_resize == False){
     return;
   }
 //printf("move.. %s\n",getInstanceName());
   if (_floated == False){
+    if (_mouse_pressed == False){
+      return;
+    }
     if (_mouse_x - pt->x > 100){
 //printf("diff!!! %s\n",getInstanceName());
       _x_bak = _x;
@@ -933,6 +1083,17 @@ void WSCfform::onMouseMove(WSCpoint* pt){
       _mouse_pressed = False;
     }
   }else{
+    if (_mouse_pressed_resize != False){
+#if 0
+      if (pt->x > 20){
+        setProperty(WSNwidth,pt->x + 5);
+      }
+      if (pt->y > 20){
+        setProperty(WSNheight,pt->y + 5);
+      }
+      needUpdate();
+#endif
+    }else
     if ((_mouse_x > 0 && _mouse_x < 15 &&
          _mouse_y > 0 && _mouse_y < _h &&
          _orientation == WS_VERTICAL) ||
@@ -965,9 +1126,11 @@ void WSCfform::onMouseMove(WSCpoint* pt){
 void WSCfform::onMouseRelease(WSCpoint* pt){
   WSCbase::onMouseRelease(pt);
   _mouse_pressed = False;
+  _mouse_pressed_resize = False;
 
 }
 WSDprivateTimer* WSCfform::_move_timer = NULL;
+WSDprivateTimer* WSCfform::_resize_timer = NULL;
 
 void WSCfform::_move_proc(void* ptr){
   WSCfform* _this = (WSCfform*)ptr;
@@ -1003,8 +1166,8 @@ void WSCfform::_move_proc(void* ptr){
 //fflush(stdout);
     }
   }
-
 }
+
 void WSCfform::_delay_proc(WSCuchar,void* ptr){
 
   WSCfform* _this = (WSCfform*)ptr;
@@ -1022,6 +1185,8 @@ void WSCfform::_delay_proc(WSCuchar,void* ptr){
   _move_timer->startTimer();
 //printf("delay_proc timer started.. %s\n",_this->getInstanceName());
 }
+
+
 void WSCfform::_detach_dev(){
   if (_dev != NULL){
     WSClistData list = getChildren();
@@ -1118,6 +1283,7 @@ void WSCfform::setFloated(WSCbool fl ){
     _floated = fl;
   }
   WSGIappTimer()->addTriggerProc(_delay_proc,WS250MS,this);
+
 //printf("delay_proc timer registered.. %s\n",getInstanceName());
 }
 WSCbool WSCfform::getFloated(){
@@ -1149,4 +1315,14 @@ long WSCfform::execEventProc(long ev, void* data){
   }
   return WSCbase::execEventProc(ev,data);
 }
-
+long WSCfform::getPixmapStyle(){
+#ifndef BTRON
+#ifndef TE
+  return _pixmap_style;
+#else
+  return WS_DYNAMIC_PIXMAP;
+#endif
+#else
+  return WS_DYNAMIC_PIXMAP;
+#endif
+}
