@@ -38,6 +38,7 @@
 #include <WSDtimer.h>
 #include <win/WSDwinMessage.h>
 #include <win/WSDwinsocket.h>
+#include <WSCinitializeManager.h>
 
 #define COLOR_MAX 256
 WSMFclassInit(WSDwinAppDev,WSDappDev);
@@ -108,6 +109,10 @@ extern WSDcodeConvert* _code_convert_create_handler_();
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi8.ico",(void*)229);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/pcreate.ico",(void*)230);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/pload.ico",(void*)231);
+    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi40.ico",(void*)232);
+    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi41.ico",(void*)233);
+    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi42.ico",(void*)234);
+    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi43.ico",(void*)235);
 extern void _win_font_init();
     _win_font_init();
 extern void _win_color_init();
@@ -121,6 +126,7 @@ WSDwinAppDev::WSDwinAppDev(){
   _hinst = 0;
   _mfocus_dev = NULL;
   _palette = 0;
+  _wnd_proc = NULL;
 }
 WSDwinAppDev::~WSDwinAppDev(){}
 void WSDwinAppDev::setMouseFocusDev(WSDwinformDev* mdev){
@@ -180,6 +186,9 @@ long WSDwinAppDev::loadGuiPolicy(){
         if (_policy == -1){
           setGuiPolicy(val,False);
         }
+      }else
+      if (!strcmp((char*)index,"#EXPOLICY") && strcmp((char*)data,"")){
+        setExtGuiPolicy((char*)data,True);
       }
     }
     if (_policy == -1){
@@ -195,6 +204,8 @@ long WSDwinAppDev::saveGuiPolicy(){
   WSCstring str;
   str << "#POLICY\n";
   str << getGuiPolicy() << "\n";
+  str << "#EXPOLICY\n";
+  str << getExtGuiPolicyName() << "\n";
 
   return WSGFreplaceTextFile("$(HOME)/.wsrc",str,False);
 }
@@ -304,6 +315,7 @@ long WSDwinAppDev::initialize(){
 }
 
 long WSDwinAppDev::handleEvents(){
+  WSGIappInitializeManager()->execInitializeProcs();
   long ret;
   while(1){
     ret = dispatchEvent();
@@ -349,6 +361,7 @@ LRESULT CALLBACK WndProc(HWND hWnd,UINT message,WPARAM uParam,LPARAM lParam){
 //  RECT rect;
 //  RECT winsize;
 //  char buffer[32];
+dbprintf("WndProc %s:%d start hwnd=0x%x message=0x%x message=%d \n",__FILE__,__LINE__,hWnd,message,message);
   WinEvent ev;
   ev.message = message;
   ev.uParam = uParam;
@@ -360,28 +373,37 @@ LRESULT CALLBACK WndProc(HWND hWnd,UINT message,WPARAM uParam,LPARAM lParam){
   }
 WSMFtrace("grabed=0x%x\n",WSGIwinAppDev()->getGrabedWnd());
 #endif
+//printf("message ==0x%x message=%d hwnd=0x%x\n",message,message,hWnd);
   if (message == WM_TIMER && hWnd == WSGIwinAppDev()->getHWND()){
     WSDwinPrivateTimer::_timer_callback(uParam);
+dbprintf("WndProc %s:%d done1\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER && uParam == WS_WIN_EXECUTE){
     WSDwinExecute* obj = (WSDwinExecute*)lParam;
     obj->_callback();
+dbprintf("WndProc %s:%d done2\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER && uParam == WS_WIN_ACCEPT){
     _accept_send_type* data = (_accept_send_type*)lParam;
     data->hd(data->sock,(void*)data->socket,data->addr);
     delete data;
+dbprintf("WndProc %s:%d done3\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER){
     int ret = WSDwinMessage::receiveMessage((WSDwinMessage*)uParam,(void*)lParam);
     if (ret != False){
+dbprintf("WndProc %s:%d done4\n",__FILE__,__LINE__);
       return 0;
     }
   }
+#if 0
   long ptr = GetWindowLong(hWnd,GWL_USERDATA);
+#else
+  HANDLE ptr = GetProp(hWnd,"frm");
+#endif
   WSDwinformDev* frm = (WSDwinformDev*)ptr;
   if (message == WM_CHAR ||
       message == WM_KEYDOWN ||
@@ -394,6 +416,7 @@ WSMFtrace("grabed=0x%x\n",WSGIwinAppDev()->getGrabedWnd());
     msg.lParam = lParam;
     WSCbool ret = WSGIwinKeyboard()->_dispatch_key(&msg);
     if (ret == False){
+dbprintf("WndProc %s:%d done5\n",__FILE__,__LINE__);
       return 0;
     }
   }
@@ -408,20 +431,89 @@ WSMFtrace("grabed=0x%x\n",WSGIwinAppDev()->getGrabedWnd());
         WSGIwinAppDev()->setMouseFocusDev(NULL);
         mdev->onMouseOut();
       }
+dbprintf("WndProc %s:%d done6\n",__FILE__,__LINE__);
       return ret;
     }
+dbprintf("WndProc %s:%d done7\n",__FILE__,__LINE__);
     return ret;
   }
   if (frm == NULL){
-    return DefWindowProc(hWnd,message,uParam,lParam);
+    HRESULT ret1 = DefWindowProc(hWnd,message,uParam,lParam);
+dbprintf("WndProc %s:%d done0\n",__FILE__,__LINE__);
+    return ret1;
   }
   WSDwinAppDev::_event_wnd = hWnd;
+#if 0 //XXZZXXZZ
+   if (message == WM_LBUTTONDOWN ||
+       message == WM_MBUTTONDOWN ||
+       message == WM_RBUTTONDOWN ){
+     HWND fwnd = GetFocus();
+     if (fwnd != hWnd || WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+       SetFocus(hWnd);
+     }
+       SetActiveWindow(hWnd);
+       SetFocus(hWnd);
+   }
+#endif 
 //printf("WSDwinAppDev::displatch.. wnd=0x%x XXZZff1\n",hWnd);
   long ret = frm->dispatchEvent(&ev);
+
+//for IE escape key navigation...
+static long _loop_in = 0;
+  if (_loop_in == 0 && GetFocus() == hWnd &&
+      (message == WM_LBUTTONDOWN ||
+       message == WM_MBUTTONDOWN ||
+       message == WM_RBUTTONDOWN )  &&
+      WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
+    if (ret != 0){
+      ret = DefWindowProc(hWnd,message,uParam,lParam);
+    }
+    _loop_in = 1;
+    MSG _msg;
+    while(1){
+      long ret2 = PeekMessage(&_msg,NULL,0,0,PM_NOREMOVE);
+      HWND target = _msg.hwnd;
+      if (target != hWnd){
+        while(1){
+          HWND target = GetParent(target);
+          if (target == NULL){
+            break;
+          }else if (target == hWnd){
+            break;
+          }
+        }
+        if (target == NULL){ //other hwnd (not hwnd of ws)
+          _loop_in = 0;
+dbprintf("WndProc %s:%d done8\n",__FILE__,__LINE__);
+          return ret;
+        }
+      }
+      if (_msg.message == WM_DESTROY ||
+          _msg.message == WM_NCDESTROY){
+          _loop_in = 0;
+dbprintf("WndProc %s:%d done9\n",__FILE__,__LINE__);
+          return ret;
+      } 
+      ret2 = GetMessage(&_msg,NULL,0,0);
+      TranslateMessage(&_msg);
+      DispatchMessage(&_msg);
+      if ( GetFocus() != hWnd ){
+        _loop_in = 0;
+dbprintf("WndProc %s:%d done10\n",__FILE__,__LINE__);
+        return ret;
+      }
+    }
+dbprintf("WndProc %s:%d done11\n",__FILE__,__LINE__);
+    return ret;
+  }
+
   if (ret != 0){
 //WSMFtrace("displatch.. wnd=0x%x ev=%d   XXZZm1\n",hWnd,message);
-    return DefWindowProc(hWnd,message,uParam,lParam);
+    HRESULT ret1 = DefWindowProc(hWnd,message,uParam,lParam);
+dbprintf("WndProc %s:%d done11\n",__FILE__,__LINE__);
+    return ret1;
   }
+dbprintf("WndProc %s:%d done12\n",__FILE__,__LINE__);
   return 0;
 #if 0
   switch(message){
@@ -563,3 +655,9 @@ long WSDwinAppDev::setInitPrm(int iprm,char** cprm){
   _cprm = cprm;
   return WS_NO_ERR;
 }
+long WSDwinAppDev::setWndProc(void* proc){
+  _wnd_proc = proc;
+}
+void* WSDwinAppDev::getWndProc(){
+  return _wnd_proc;
+}
