@@ -159,9 +159,16 @@ long WSDwinKeyboard::resetFep(WSDdev* dev){
   return WS_NO_ERR;
 }
 
+WSDdev* _fep_dev = NULL;
+short _fep_pos_x = -1;
+short _fep_pos_y = -1;
+short _fep_font_no = -1;
+short _fep_fg_no = -1;
+short _fep_bg_no = -1;
 
 long WSDwinKeyboard::setFepPos(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
 WSMFtrace("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
+//printf("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
   HWND wnd = (HWND)dev->getWindowResource();
   if (wnd == NULL || wnd == (HWND)-1){
     return WS_ERR;
@@ -184,7 +191,14 @@ WSMFtrace("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
   cf.ptCurrentPos.x = sx;
   cf.ptCurrentPos.y = sy;
   ImmSetCompositionWindow(_himc,&cf);
+  _fep_dev = dev;
+  _fep_pos_x = x;
+  _fep_pos_y = y;
+  _fep_font_no = fontno;
+  _fep_fg_no = fgno;
+  _fep_bg_no = bgno;
 WSMFtrace("WSDwinKeyboard::setFepPos()  fontno=%d fno=%d\n",fontno,_font_no);
+//printf("WSDwinKeyboard::setFepPos()  fontno=%d fno=%d\n",fontno,_font_no);
   if (fontno != _font_no){
     WSDfont* fn = WSGIappFontSet()->getFont(fontno);
     if (fn == NULL){
@@ -204,15 +218,36 @@ WSMFtrace("WSDwinKeyboard::setFepPos() setfl=0x%x\n",lf);
 }
 
 WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
-
+  if (msg->wParam ==VK_PROCESSKEY){
+//printf("process key ev=%d\n",msg->message);
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCKR &&
+        _fep_dev != NULL && msg->message != WM_KEYUP){
+      setFepPos(_fep_dev,_fep_pos_x,_fep_pos_y,_fep_font_no,_fep_fg_no,_fep_bg_no);
+    }else
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCCN &&
+        _fep_dev != NULL && msg->message != WM_KEYUP){
+      setFepPos(_fep_dev,_fep_pos_x,_fep_pos_y,_fep_font_no,_fep_fg_no,_fep_bg_no);
+    }
+    return False;
+  }
+//printf("here001 ev=%d code=0x%x\n",msg->message,msg->wParam);
   if (msg->message == WM_CHAR || msg->message == WM_SYSCHAR){
-//printf("here001\n");
+    setIsPressed(True);
+//printf("here001 code=0x%x\n",msg->wParam);
     if (GetKeyState(VK_CONTROL) <0){
       if (msg->message == WM_CHAR ){
+//printf("here001 return..1\n");
         return True;
       }
       return False;
     }
+    if (msg->wParam == VK_CANCEL ||
+        msg->wParam == VK_BACK ||
+        msg->wParam == VK_TAB ||
+        msg->wParam == VK_RETURN ||
+        msg->wParam == VK_ESCAPE ){
+      return False;
+    }
     if (msg->wParam != VK_CANCEL &&
         msg->wParam != VK_BACK &&
         msg->wParam != VK_TAB &&
@@ -221,12 +256,20 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
       if (_pos == 0){
         _buf[_pos] = (char)msg->wParam;
         _buf[_pos+1] = 0;
-        if ((msg->wParam >= 0x80 && msg->wParam < 0xa0 ) ||
-             msg->wParam > 0xdf){
+        if ((((msg->wParam >= 0x80 && msg->wParam < 0xa0 ) ||
+             msg->wParam > 0xdf) && 
+        WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_SJIS) ||
+        ((msg->wParam & 0x80 && msg->wParam < 0xff) && 
+        WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCKR) ||
+        ((msg->wParam & 0x80 && msg->wParam < 0xff) && 
+        WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCCN)){
           _pos = 1;
+//printf("here001 return..2\n");
+          setKey(0);
           return True;
         }else{
 WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
+//printf("key pressed key=#%s#  XXZZk1 xx\n",_buf);
           setText(_buf,WS_EN_LOCALE);
           _is_tty_function = False;
           if (_buf[0] == '!'){
@@ -358,6 +401,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
 //          _buf[0] = 0;
 //        }
 WSMFtrace("key pressed key=#%s#  XXZZk1 xx2\n",_buf);
+//printf("key pressed2 key=#%s#  XXZZk1 xx2 pos=%d\n",_buf,_pos);
         setText(_buf,WS_EN_LOCALE);
         _pos = 0;
         _is_tty_function = False;
@@ -380,6 +424,7 @@ WSMFtrace("key pressed getkey=0x%x  XXZZk1\n",getKey());
         _with_shift = False;
       }
 WSMFtrace("key pressed shift=%d alt=%d XXZZk1\n",_with_shift,_with_alt);
+//printf("key pressed executed.. shift=%d alt=%d XXZZk1\n",_with_shift,_with_alt);
 
       WSCbool executed = executeHandler(True);
       if (executed != False){
@@ -389,6 +434,8 @@ WSMFtrace("key pressed shift=%d alt=%d XXZZk1\n",_with_shift,_with_alt);
       }
     }
   }else if (msg->message == WM_KEYDOWN){
+    setIsPressed(True);
+//printf("keydown key=0x%x  encode=%d\n",msg->wParam,WSGIappLocaleSet()->getSystemLocaleEncoding());
 //printf("here01 0x%x 0x%x\n",msg->wParam,msg->lParam);
 WSMFtrace("key down setkey=0x%x  XXZZk141\n",msg->wParam);
     if (GetKeyState(VK_CONTROL) < 0){
@@ -544,6 +591,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
         setKey(key1);
       }
 WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
+//printf("key pressed executed2.. setkey=0x%x  #%s# encode=%d\n",msg->wParam,_buf,WSGIappLocaleSet()->getSystemLocaleEncoding());
       _buf[0] = 0;
       setText(_buf,WS_EN_LOCALE);
       _with_cntl = True;
@@ -647,6 +695,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
         }else{
           _is_tty_function = False;
         }
+//printf("executed3.. buf=#%s#\n",_buf);
         WSCbool dispatched = executeHandler(True);
         if (dispatched != False){
           return False;
@@ -761,6 +810,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
           setKey(key1);
         }
 WSMFtrace("key pressed setkey=0x%x  XXZZk142\n",msg->wParam);
+//printf("key pressed4 setkey=0x%x  XXZZk142\n",msg->wParam);
         if ( msg->wParam == WSK_Left ||
              msg->wParam == WSK_Up ||
              msg->wParam == WSK_Right ||
@@ -789,6 +839,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk142\n",msg->wParam);
       }
     }
   }else if (msg->message == WM_KEYUP){
+    setIsPressed(False);
     if (msg->wParam == VK_MENU){
       _with_alt = False;
     }
@@ -821,8 +872,9 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk142\n",msg->wParam);
       return True;
     }
   }else if (msg->message == WM_SYSKEYDOWN){
+    setIsPressed(True);
     long key1 = (long)msg->wParam;
-//printf("key4=0x%x %d\n",key1,key1);
+//printf("key6=0x%x %d\n",key1,key1);
     if (key1 == '!'){
       setKey(WSK_exclam);
     }else
@@ -963,6 +1015,7 @@ WSMFtrace("key pressed XXZZk13 0x%x\n",getKey());
     }
 #endif
   }else if (msg->message == WM_SYSKEYUP){
+    setIsPressed(False);
 WSMFtrace("key released  XXZZk14 0x%x\n",msg->wParam);
     if (msg->wParam == VK_MENU){
       _with_alt = False;
