@@ -55,6 +55,7 @@ static _WSDwinformDev_init_ _init_to_run_WSDwinformDev_;
 static long _os_flag = -1;
 
 WSDwinformDev::WSDwinformDev(){
+dbprintf("WSDwinformDev::WSDwinformDev %s:%d start\n",__FILE__,__LINE__);
   _click_time = 0;
   if (_os_flag == -1){
     OSVERSIONINFO osvi;
@@ -84,15 +85,18 @@ WSDwinformDev::WSDwinformDev(){
 //      draw_of_winform = new WSDwindraw();
 //  }
 //  setDraw(draw_of_winform);
+dbprintf("WSDwinformDev::WSDwinformDev %s:%d done\n",__FILE__,__LINE__);
 }
 
 WSDwinformDev::~WSDwinformDev(){
+dbprintf("WSDwinformDev::~WSDwinformDev %s:%d start\n",__FILE__,__LINE__);
 //printf("WSDwinformDev::~WSDwinformDev this=0x%x XXZZff1\n",this);
 //WSMFtrace("WSDwinformDev::~WSDwinformDev 0x%x\n",this);
 //  destroyContext();
   destroyPixmap();
   destroyWindow();
   setExposed(False);
+dbprintf("WSDwinformDev::~WSDwinformDev %s:%d done\n",__FILE__,__LINE__);
 }
 void WSDwinformDev::setExposed(WSCbool fl){
   if (fl != False){
@@ -155,6 +159,9 @@ long WSDwinformDev::setValue(long kind,void* data){
 //printf(" -> %d  w=0x%d\n",p.left,p.right - p.left);
              _w_x = *(short*)data;
              MoveWindow(_wnd,_w_x,_w_y,p.right,p.bottom, True);
+             if (getVisible() != False){
+               _expose_children(0,0,_w_w,_w_h);
+             }
              break;
     case WSDEV_Y:
              if (_wnd == NULL){
@@ -166,6 +173,9 @@ long WSDwinformDev::setValue(long kind,void* data){
              _w_y = *(short*)data;
              MoveWindow(_wnd,_w_x,_w_y,p.right,p.bottom, True);
 //printf(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
+             if (getVisible() != False){
+               _expose_children(0,0,_w_w,_w_h);
+             }
              break;
     case WSDEV_WIDTH:
              if (_wnd == NULL){
@@ -605,8 +615,15 @@ extern char szAppName[];
   _pwnd = windev->getHWND();
   _wnd = CreateWindow( szAppName,"",WS_CHILD | WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,x,y,w,h,windev->getHWND(),
                         NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
+  if (WSGIwinAppDev()->getWndProc() != NULL){
+    SetWindowLong(_wnd,GWL_WNDPROC,(LONG)WSGIwinAppDev()->getWndProc());
+  }
 //WSMFtrace("x,y,w,h=%d,%d,%d,%d\n",x,y,w,h);
+#if 0
   SetWindowLong(_wnd,GWL_USERDATA,(long)this);
+#else
+  SetProp(_wnd,"frm",(HANDLE)this);
+#endif
 //WSMFtrace("WSDwinformDev  this=0x%x wnd=0x%x    pwnd=0x%x\n",this,_wnd,windev->getHWND());
 #ifdef DEBUG
 WSMFtrace("WSDwinformDev::createWindow() add callbacks is not implemented..\n");
@@ -826,7 +843,11 @@ WSMFtrace("WSDwinformDev::destroyWindow() remove callbacks is not implemented..\
     WSGIwinAppDev()->setMouseFocusDev(NULL);
   }
   if (_wnd != 0){
+#if 0
     SetWindowLong(_wnd,GWL_USERDATA,(long)0);
+#else
+    SetProp(_wnd,"frm",(HANDLE)0);
+#endif
     MSG msg;
     while(PeekMessage(&msg,_wnd,0,0,PM_REMOVE)){
       if (msg.message == WM_PAINT){
@@ -1001,6 +1022,41 @@ long WSDwinformDev::dispatchEvent(WinEvent* e){
    if (e->message == WM_DESTROY){
      _evh_delete(e);
      return 0;
+   }else
+   if (e->message == WM_KILLFOCUS){
+     WSCbase* fcl = WSCbase::getFocusInstance();
+     if (fcl != NULL){
+       fcl->onWindowFocusChange(False);
+     }
+     return 0;
+   }else
+   if (e->message == WM_SETFOCUS){
+     WSCbase* fcl = WSCbase::getFocusInstance();
+     if (fcl != NULL){
+       WSCbase* cl = getAttachedClient();
+       WSCbase* pcl = fcl;
+       if (cl != NULL){
+         WSCbool fl = False;
+         while(1){
+           if (pcl == NULL){
+             break;
+           }
+           if (cl == pcl){
+             fl = True;
+             break;
+           }
+           pcl = pcl->getParent();
+         }
+         if (fl != False){
+           fcl->onWindowFocusChange(True);
+         }else{
+           fcl->setFocus(False);
+         }
+       }else{
+         fcl->onWindowFocusChange(True);
+       }
+     }
+     return 0;
    }
    return 1;
 }
@@ -1273,6 +1329,7 @@ void WSDwinformDev::_evh_dummy(WinEvent*){
 }
 
 void WSDwinformDev::_evh_delete(WinEvent* ev){
+dbprintf("WSDwinformDev::_evh_delete %s:%d start\n",__FILE__,__LINE__);
   _pwnd = 0;
   _wnd = 0;
   setInitialized(False);
@@ -1285,16 +1342,19 @@ void WSDwinformDev::_evh_delete(WinEvent* ev){
     }
   }
 #endif
+dbprintf("WSDwinformDev::_evh_delete %s:%d done\n",__FILE__,__LINE__);
 }
 
 
 void WSDwinformDev::_evh_delete2(void* ptr){
+dbprintf("WSDwinformDev::_evh_delete2 %s:%d start\n",__FILE__,__LINE__);
    WSDwinformDev* dev = (WSDwinformDev*)ptr;
 
    dev->_pwnd = 0;
    dev->_wnd = 0;
    dev->setInitialized(False);
 //   dev->detachClient();
+dbprintf("WSDwinformDev::_evh_delete2 %s:%d done\n",__FILE__,__LINE__);
 }
 
 void WSDwinformDev::setHWND(HWND w){
@@ -1375,3 +1435,39 @@ long WSDwinformDev::lower(){
   SetWindowPos(_wnd,HWND_BOTTOM,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
   return WS_NO_ERR;
 }
+
+void WSDwinformDev::_expose_children(int x,int y,unsigned long w,unsigned long h){
+  WSCbase* client = getAttachedClient();
+  if (client == NULL){
+    return;
+  }
+
+  WSClistData list = client->getChildren();
+  long i;
+  long num = list.getNum();
+  for(i=0; i<num; i++){
+    WSCbase* child = (WSCbase*)list[i];
+    if (child->getVisible() == False){
+      continue;
+    }
+    WSDdev* dev = child->getdev();
+    if (dev == NULL){
+      continue;
+    }
+    WSDwinformDev* wdev =(WSDwinformDev*)dev->cast("WSDwinformDev");
+    if (wdev == NULL){
+      continue;
+    }
+    long ex,ey;
+    unsigned long ew,eh;
+    short dx,dy;
+    unsigned short dw,dh;
+    wdev->getWindowSize(&dx,&dy,&dw,&dh);
+    WSGFandArea(0,0,w,h,dx,dy,dw,dh,
+                &ex,&ey,&ew,&eh);
+    if (ew != 0 && eh != 0){
+      wdev->exposeArea(ex-dx,ey-dy,ew,eh);
+      wdev->_expose_children(ex-dx,ey-dy,ew,eh);
+    }
+  }
+}
