@@ -129,6 +129,10 @@ printf("XXZZlang2 0x%x\n",GetSystemDefaultLCID());
     xu_locale_encoding = XU_CONV_SJIS;
   }else if ((lid & 0x3ff) == LANG_KOREAN){
     xu_locale_encoding = XU_CONV_EUCKR;
+  }else if ((lid & 0x3ff) == LANG_CHINESE){
+    if (SUBLANGID(lid) == SUBLANG_CHINESE_SIMPLIFIED){
+      xu_locale_encoding = XU_CONV_EUCCN;
+    }
   }else{
     xu_locale_encoding = XU_CONV_LOCALE;
   }
