@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -704,6 +704,10 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
     return WS_NO_ERR;
   }
 
+  if ((long)pix1 != -1 && (long)pix1 != 0){
+    XSetClipMask(_display,_gc,0);
+  }
+
   XImage* img = (XImage*)image->getValue3();
   if ((long)img == 0 ||
       (long)img == -1 ){
@@ -717,14 +721,17 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
       return WS_ERR;
     }
   }
-
-  if ((long)pix1 != -1 && (long)pix1 != 0){
-    XSetClipMask(_display,_gc,0);
+  XImage* img_mask = NULL;
+  if (image->cast("WSDximage")){
+    WSDximage* ximage = (WSDximage*)image->cast("WSDximage");
+     img_mask = ximage->createMaskImage();
+//printf("img_mask->bytes_per_line=%d\n",img_mask->bytes_per_line);
+//printf("img_mask->depth=%d\n",img_mask->depth);
   }
-
   long display_depth = XDefaultDepth(_display,
                                      XDefaultScreen(_display));
 //printf("display_depth=%d\n",display_depth);
+  WSDxappDev* app   = WSGIxwinAppDev(); 
   long bit_pad = 32;
   if (display_depth == 32){
      bit_pad = 32;
@@ -745,16 +752,28 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
                        ZPixmap,0,0,
                        dw, dh, bit_pad,0);
   XInitImage(img2);
+  int msb_flag = (ImageByteOrder(_display) == MSBFirst)?1:0;
   WSCulong iw = image->getImageWidth();
   WSCulong ih = image->getImageHeight();
   img2->data = new char[img2->bytes_per_line * dh ];
+//printf("img2->bytes_per_line=%d\n",img2->bytes_per_line);
   long bit = img2->bits_per_pixel;
 
-
-//  if ((long)pix2 != -1 && pix2 != 0){
-//    XSetClipMask(_display,_gc,pix2);
-//    XSetClipOrigin(_display,_gc,x,y);
-//  }
+  XImage* img_mask2 = NULL;
+  WSCulong iwm = 0;
+  WSCulong iwm2 = 0;
+//  WSCulong ihm = image->getImageHeight();
+  if (img_mask != NULL){
+    img_mask2 = XCreateImage(_display, visual,
+                       1,
+                       ZPixmap,0,0,
+                       dw, dh,8,0);
+    XInitImage(img_mask2);
+    img_mask2->data = new char[img_mask2->bytes_per_line * dh ];
+    iwm = img_mask->bytes_per_line;
+    iwm2 = img_mask2->bytes_per_line;
+//printf("img_mask2->bytes_per_line=%d\n",img_mask2->bytes_per_line);
+  }
 
   long i=0;
   long j=0;
@@ -821,16 +840,62 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
         ptr = ((j -dx +x)+(i -dy +y)*img2->bytes_per_line);
         img2->data[ptr++] = img->data[pt];
       }
+      if (img_mask2 != NULL){
+        pt = (long)((double)j*iw/w)/8 + (long)((double)i*ih/h)*iwm;
+        ptr = ((j -dx +x)/8+(i -dy +y)*img_mask2->bytes_per_line);
+        WSCuchar val = img_mask2->data[ptr];
+        long shiftd = (j -dx +x)%8;
+        long shifts = ((int)((double)j*iw/w))%8;
+        WSCuchar val2 = img_mask->data[pt];
+        WSCuchar m = 1;
+        if (!msb_flag){
+          m <<= shifts;
+        }else{
+          m =0x80;
+          m >>= shifts;
+        }
+        if (val2 & m){
+          WSCuchar m = 1;
+          m <<= shiftd;
+//printf("ptr=%d %d,pt=%d %d %d 0x%x\n",ptr,shiftd,pt,shiftd,val2&m,m);
+          val |= m;
+        }else{
+          WSCuchar m = 1;
+          m <<= shiftd;
+//printf("ptr=%d %d,pt=%d %d %d 0x%x\n",ptr,shiftd,pt,shiftd,val2&m,~m);
+//TEST
+          val &= ~m;
+        }
+        img_mask2->data[ptr] = val;
+      }
     }
   }
   Pixmap newpix = WSGFxGetPixmap(dw,dh);
 //  XPutImage(_display,newpix,_gc,img2,dx-x,dy-y,0,0,dw,dh);
   XPutImage(_display,newpix,_gc,img2,0,0,0,0,dw,dh);
 
+  Pixmap pixm = 0;
+  if (img_mask2 != NULL){
+    pixm = XCreateBitmapFromData(XtDisplay(app->appWidget()),
+                             XtWindow(app->appWidget()),img_mask2->data,dw,dh);
+
+    if ((long)pixm != -1 && pixm != 0){
+      XSetClipMask(_display,_gc,pixm);
+      XSetClipOrigin(_display,_gc,dx,dy);
+    }
+  }
+
+
 //  XCopyArea(_display,newpix,_window,_gc, dx -x,dy -y, dw,dh,dx,dy);
   XCopyArea(_display,newpix,_window,_gc, 0,0, dw,dh,dx,dy);
 
   WSGFxReleasePixmap(newpix);
+  if ((long)pixm != -1 && pixm != 0){
+//    XSetClipMask(_display,_gc,0);
+    XDestroyImage(img_mask);
+    XDestroyImage(img_mask2);
+    XFreePixmap(XtDisplay(app->appWidget()),pixm);
+  }
   if (img2->data != NULL){
     delete img2->data;
     img2->data = NULL;
