@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -351,11 +351,12 @@ long WSDxscrFrameDev::createWindow(){
   XtAddEventHandler(_widget,LeaveWindowMask,False,(XtEventHandler)_evh_mouse_out,(WSDxformDev*)this);
   XtAddEventHandler(_widget,KeyPressMask | KeyReleaseMask,False,(XtEventHandler)_evh_dummy,(WSDxformDev*)this);
   XtAddCallback(_widget,XtNdestroyCallback,_evh_delete2,this);
-
+//printf("WSDxscrFrameDev::createWindow this=0x%x _widget=0x%x _scr_form=0x%x\n",this,_widget,_scr_form);
   return WS_NO_ERR;
 }
 
 long WSDxscrFrameDev::destroyWindow(){
+//printf("WSDxscrFrameDev::destroyWindow this=0x%x _widget=0x%x _scr_form=0x%x\n",this,_widget,_scr_form);
   if (_widget != 0){
     XtRemoveEventHandler(_widget,ExposureMask,False,(XtEventHandler)_evh_expose,(WSDxformDev*)this);
     XtRemoveEventHandler(_widget,ButtonPressMask,False,(XtEventHandler)_evh_btn_press,(WSDxformDev*)this);
@@ -371,12 +372,7 @@ long WSDxscrFrameDev::destroyWindow(){
     XtRemoveEventHandler(_scr_form,StructureNotifyMask,False,(XtEventHandler)_evh_resize,this);
     XtDestroyWidget(_scr_form);
   }
-  _pwidget = 0;
-  _widget = 0;
-  _scr_form = 0;
-//  setDependDev(NULL);
-  setInitialized(False);
-//  destroyAllDependChildren();
+  _clear_widget();
   return WS_NO_ERR;
 }
 
@@ -416,17 +412,19 @@ long WSDxscrFrameDev::setVisible(WSCbool fl){
   }
   return WS_NO_ERR;
 }
-
+void WSDxscrFrameDev::_clear_widget(){
+  _pwidget = 0;
+  _scr_form = 0;
+  _widget = 0;
+  setInitialized(False);
+}
 void WSDxscrFrameDev::_evh_delete2(Widget w,XtPointer ptr,XtPointer ){
    WSDxscrFrameDev* dev = (WSDxscrFrameDev*)ptr;
-
+//printf("WSDxscrFrameDev::_evh_delete2 this=0x%x w=0x%x _widget=0x%x\n",dev,w,dev->_widget);
    if (dev->_widget != w){
      return;
    }
-   dev->_pwidget = 0;
-   dev->_scr_form = 0;
-   dev->_widget = 0;
-   dev->setInitialized(False);
+   dev->_clear_widget();
 }
 void WSDxscrFrameDev::_evh_resize(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
   WSGIxwinAppDev()->setXEvent(ev);
