@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2004 Toshikaz Hirabayashi
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -329,6 +329,15 @@ WSMFtrace("WSCxwinDev::setValue height=%d _w_h=%d\n",*(WSCushort*)data,_w_h);
              }else{
                return WS_ERR;
              }
+    case WSDEV_MOUSE_NO:
+             _mno = *(WSCushort*)data;
+             if (_widget != NULL && XtWindow(_widget) != 0){
+               long val = WSGIappMouse()->getMouseCursor(_mouse_no);
+               if (val != 0){
+                 XDefineCursor(XtDisplay(_widget),XtWindow(_widget),val);
+               }
+             }
+             return WS_ERR;
     default:
       return WS_ERR;
   }
@@ -430,8 +439,8 @@ long WSDxwinDev::createWindow(){
 
 long WSDxwinDev::destroyWindow(){
 //printf("WSCxwinDev::destroyWindow dev=0x%x _swidget=0x%x vis=%d\n",this,_swidget,getVisible());
+  WSCbool destroy = False;
   if (_widget != 0){
-    clearChildWidgets();
     XtRemoveEventHandler(_widget,ExposureMask,False,(XtEventHandler)_evh_expose,this);
     XtRemoveEventHandler(_widget,ButtonPressMask,False,(XtEventHandler)_evh_btn_press,(WSDxformDev*)this);
     XtRemoveEventHandler(_widget,ButtonReleaseMask,False,(XtEventHandler)_evh_btn_release,(WSDxformDev*)this);
@@ -443,7 +452,7 @@ long WSDxwinDev::destroyWindow(){
 #if 1 //XUNICODE
     XUFreeInfoWindow(XtDisplay(_widget),XtWindow(_widget));
 #endif //UNICODE
-
+    destroy = True;
     XtDestroyWidget(_widget);
     _widget = 0;
   }
@@ -455,6 +464,9 @@ long WSDxwinDev::destroyWindow(){
     XtDestroyWidget(_swidget);
     _swidget = 0;
   }
+  if (destroy != False){
+    clearChildWidgets();
+  }
   setInitialized(False);
   WSGIxwinAppDev()->dispatchEvent();
   if (_sizehints != NULL){
@@ -486,7 +498,11 @@ void WSDxwinDev::_float_top_timer_proc(WSCuchar,void* data){
   }
 
 }
-
+void WSDxwinDev::_clear_widget(){
+  _swidget = 0;
+  _widget = 0;
+  setInitialized(False);
+}
 long WSDxwinDev::setVisible(WSCbool fl){
 WSMFtrace("WSCxwinDev::setVisible _w_h=%d\n",_w_h);
   WSCbool bk = getVisible();
@@ -507,9 +523,6 @@ WSMFtrace("WSCxwinDev::setVisible _w_h=%d\n",_w_h);
         XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
 #endif
         clearChildWidgets();
-        _swidget = 0;
-        _widget = 0;
-        setInitialized(False);
         onUnmap();
         if (_sizehints != NULL){
           XFree(_sizehints);
@@ -577,7 +590,38 @@ void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
     return;
   }
   if (ev->type == FocusIn){
-    dev->onFocusChange();
+    WSCbase* fcl = WSCbase::getFocusInstance();
+    if (fcl != NULL){
+      WSCbase* cl = dev->getAttachedClient();
+      WSCbase* pcl = fcl;
+      if (cl != NULL){
+        WSCbool fl = False;
+        while(1){
+          if (pcl == NULL){
+            break;
+          }
+          if (cl == pcl){
+            fl = True;
+            break;
+          }
+          pcl = pcl->getParent();
+        }
+        if (fl != False){
+          fcl->onWindowFocusChange(True);
+        }else{
+          fcl->setFocus(False);
+        }
+      }else{
+        fcl->onWindowFocusChange(True);
+      }
+    }else{
+      dev->onFocusChange();
+    }
+  }else if (ev->type == FocusOut){
+    WSCbase* fcl = WSCbase::getFocusInstance();
+    if (fcl != NULL){
+      fcl->onWindowFocusChange(False);
+    }
   }
 }
 
@@ -603,12 +647,7 @@ void WSDxwinDev::_evh_delete2(Widget w,XtPointer ptr,XtPointer ){
    if (dev->_widget != w){
      return;
    }
-
-   dev->_swidget = 0;
-   dev->_widget = 0;
-//   dev->setDependDev(NULL);
-   dev->setInitialized(False);
-//   dev->destroyAllDependChildren();
+   dev->_clear_widget();
    dev->onUnmap();
 }
 
